# PE RBAC Service

_Being Trapper Keeper service definitions for PE's RBAC system and API_.

## Development

### Dev Database setup

Ensure that you have Postgresql installed and running.
You'll want to run something like the following:

        $ sudo su postgres
        $ psql
        psql (9.2.7)
        Type "help" for help.

        postgres=# create user perbac with password 'foobar';
        CREATE ROLE

        postgres=# create database perbac;
        CREATE DATABASE

        postgres=# grant all on database perbac to perbac;
        GRANT

and then edit your pg_hba.conf (on ubuntu:
`/etc/postgresql/9.1/main/pg_hba.conf`, on mac try `/usr/local/var/postgres/pg_hba.conf`) to add

        local all perbac password

finally, restart postgresql (however your platform wants) to reflect
the pg_hba.conf change (the restart might not be necessary).

### Migrations

We use [migratus](https://github.com/pjstadig/migratus). Check there
for more detailed information, but for now ensure that you have run:

        lein migratus migrate

In case a migration you have already run once is changed and you need
to re-run it, do:

        lein migratus down <migration id>
        lein migratus migrate

to re-apply. As a courtesy to fellow developers, please be sure to
announce when changing existing migrations that have been pushed out
(so others can reset their database).

## Reset the DB from the command line

There is a single purpose TK service that exists to reset the database
and fully migrate it. It's callable as a lein alias:

        lein initdb

### Running the app

If you just want to run the app, run

    lein rbac

This is aliased to `lein trampoline run --config dev-resources/conf.d` in `project.clj`.


You can contact the API endpoints at

    http://localhost:8080/rbac-api/v1

So, for example, the `/roles` endpoint can be reached at
`http://localhost:8080/rbac-api/v1/roles`.

For more info on the API, see the [API
documentation](doc/api.md).

### Admin Account

By default, the admin account's user name is 'admin' and the password
is 'admin'. The password will eventually be set as part of the
installer but for local development, it will be set to 'admin'.

### REPL integration

If you're going to be developing on the RBAC service, you will probably want
to run it throught the REPL. This will allow you to interact with it more
easily. REPL integration also has support for reloading the services without
restarting the JVM,  which will allow you to see your changes reflected more
quickly.

Run

    lein repl

The tools to run the RBAC service from the repl are automtically loaded (through
`repl-options` in `project.clj`) when you start the repl.

You should then be able to run `(go)` to launch the jetty webserver and
services, `(reset)` to reload, and `(stop)` to shutdown.

### Fixtures

Tools to load fixture data are included in the `puppetlabs.rbac.testutils.db`
and `puppetlabs.rbac.testutils.fixtures` namespaces. To load the fixtures into
the DB, start up the repl (which automatically requires these namespaces
through `repl-options` in `project.clj`), then run `(load-fixtures!)`. The DB
to load fixtures into can be changed by supplying a config file location as an
argument to load-fixtures.  Fixtures can be reset with `(reset-fixtures!)`. To
delete everything in the DB, use `(clear-db!)`. Both of these optionally take
a config file path argument, like `(load-fixtures!
"dev-resources/conf.d")`.

### Tests

#### Setup

After you've set up the dev database, you can also set up the database needed
for integration testing, `perbac_test`, as follows:

        postgres=# create database perbac_test;
        CREATE DATABASE

        postgres=# grant all on database perbac_test to perbac;
        GRANT

To run all tests, run

    lein test :all

You can run only unit tests with

    lein test :unit

Or only integration tests:

    lein test :integration

Integration tests are marked using a test selector after `deftest`.
Tests that have no test selector are assumed to be unit tests.
For more information on test selectors, check out `lein test --help`.

You can run an individual namespace of tests with, e.g.

    lein test :only puppetlabs.rbac.storage-integration-test

(or the namespace of your choice)

### Deploying Snapshots and Making Releases

Deploying snapshots to the internal Nexus server requires that you have a login established on [nexus.delivery.puppetlabs.net](http://nexus.delivery.puppetlabs.net/). Ask Chris Price to set up an account for you. Then you can deploy a snapshot (use your LDAP credentials when prompted):

    lein deploy

To make a release, [build with parameters on the RBAC Service Internal Release Jenkins bob](http://jenkins-enterprise.delivery.puppetlabs.net/job/RBAC%20Service%20Internal%20Release/build?delay=0sec)

Copyright © 2014 Puppet Labs, inc.
